@@ -10,7 +10,7 @@
 # if sort is T, data will have rows in the same order
 # as the taxon names in phy$tip.label
 
-treedata<-function(phy, data, data.names=NULL, sort=F)
+treedata<-function(phy, data, data.names=NULL, sort=F, warnings=T)
 {
 
 	if(is.vector(data)) data<-as.matrix(data)
@@ -20,7 +20,8 @@ treedata<-function(phy, data, data.names=NULL, sort=F)
 	if(is.null(data.names)) {
 		if(is.null(rownames(data))) {
 				data.names<-phy$tip.label[1:dim(data)[1]]
-				cat("Warning: no tip labels, order assumed to be the same as in the tree\n")
+				if(warnings)
+					cat("Warning: no tip labels, order assumed to be the same as in the tree\n")
 			} else
 				data.names<-rownames(data)
 	}
@@ -28,20 +29,23 @@ treedata<-function(phy, data, data.names=NULL, sort=F)
 	if(is.na(nc[[1]][1]) | nc[[1]][1]!="OK") {
 		if(length(nc[[1]]!=0)) {
 			phy=drop.tip(phy, nc[[1]])
-			cat("Dropped tips from the tree because there were no matching names in the data:\n")
-			print(nc[[1]])
-			cat("\n")
+			if(warnings) {
+				cat("Dropped tips from the tree because there were no matching names in the data:\n")
+				print(nc[[1]])
+				cat("\n")
+			}
 		}
 	
 		if(length(nc[[2]]!=0)) {
 			m<-match(data.names, nc[[2]])
 			data=as.matrix(data[is.na(m),])
 			data.names<-data.names[is.na(m)]
-			cat("Dropped rows from the data because there were no matching tips in the tree:\n")
-			print(nc[[2]])
-			cat("\n")
-
+			if(warnings) {
+				cat("Dropped rows from the data because there were no matching tips in the tree:\n")
+				print(nc[[2]])
+				cat("\n")
 			}
+		}
  	}
 	order<-match(data.names, phy$tip.label)	
 
